#!/bin/bash
#debugging output
[ -f /tmp/debug ] && set -x

#this script will make a disk image with the tools needed to update your Recovery HD with the newest OS X release
# Author: Joel Bruner (aka the brunerd) 2012

#Changelog
# ver 15.2.28 - Creates an additional script in the DMG, "RecoveryHD PKG Maker.command" which will make a flat pkg with custom bundle name, version, and can be signed also
# ver 12.25.7 - Allows ~ paths to be typed in now, OS version appears in dmg name now, quieter, takes ReocoveryHDUpdate.dmg or pkg, changed verbage to accomodate Lion and Mountain Lion
# ver 1.0 - Initial Release

#Required components:
#Lion Recovery Update v1.0 - http://support.apple.com/downloads/DL1464/en_US/RecoveryHDUpdate.dmg
#"Install Mac OS X Lion.app" - App Store


###########################
# VARIABLES
###########################

#IMAGETEMP is the pathname for the disk image being built, will the OS and Build appended to the name later
IMAGETEMP="/tmp/Recovery HD Updater"

#temp folder for package expansion
RECOVERY_EXPANDED="/tmp/RecoveryHDUpdate"

#Mounted disk image paths 
ESDPATH="/Volumes/Mac OS X Install ESD"
RECOVERYPATH="/Volumes/Mac OS X Lion Recovery HD Update"
BASESYSTEMPATH="/Volumes/OS X Base System"

###########################
# FUNCTIONS
###########################

#call this later when OS Version is known and can be used in the package name
function createRecoveryScripts
{

#this script that interactively updates a disk
RecoveryHDUpdaterScript='#!/bin/bash
#brunerd.com 2012
#debugging output
[ -f /tmp/debug ] && set -x

clear;
MYPATH="$(dirname "$0")"

#get destination drive
if [ "$1" == "" ]; then
	echo -n "Please drag in DESTINATION disk for Recovery HD partition and press Enter: "
	while [ -z "$DEST" ]; do
	read DEST
	done
else
	DEST="$1"
fi

if [ ! -d "$DEST" ]; then 
	echo "$DEST not found"
	exit
else
	#create Recovery partition
	sudo "$MYPATH"/bin/dmtest ensureRecoveryPartition "$DEST" "$MYPATH"/etc/BaseSystem.dmg 0 0 "$MYPATH"/etc/BaseSystem.chunklist
fi
'

#this is the script that makes the PKG
RecoveryHDUpdaterPKGScript='#!/bin/bash
#brunerd.com 2012
#debugging output
[ -f /tmp/debug ] && set -x

clear;

buildPath="/private/tmp/RecoveryHDUpdater_BuildTemp"
payloadPath="$buildPath/payload/"
scriptPath="$buildPath/scripts"

postinstallScript='"'"'#!/bin/bash

MYPATH="/private/tmp/RecoveryHDUpdater"
sudo "$MYPATH"/bin/dmtest ensureRecoveryPartition / "$MYPATH"/etc/BaseSystem.dmg 0 0 "$MYPATH"/etc/BaseSystem.chunklist

rm -rf "$MYPATH"
'"'"'

#get some other details
echo -n "PKG Receipt Bundle ID? [default: com.nobody.recoveryhdupdate]: "
		read PKGID

echo -n "Signing ID? [Enter for No]: "
		read SIGNINGID


#make temp folders
mkdir -p "$buildPath"
mkdir -p "$scriptPath"
mkdir -p "$payloadPath"

echo "Copying ./etc and ./bin"
MYPATH="$(dirname "$0")"
#copy to new folder
cp -R "$MYPATH"/bin "$payloadPath"
cp -R "$MYPATH"/etc "$payloadPath"

echo "Creating postinstall"
echo "$postinstallScript" > "$scriptPath"/postinstall
chmod ugo+x "$scriptPath"/postinstall

echo "Creating " ~/Desktop/RecoveryHDUpdater'_${OSVER}_${OSBUILD}'.pkg, PackageVersion: '$OSVER', PackageIdentifier: ${PKGID:="com.nobody.recoveryhdupdate"}

if [ -n "$SIGNINGID" ]; then
pkgbuild --root "$payloadPath" --identifier $PKGID --scripts "$scriptPath" --version '$OSVER' --install-location /private/tmp/RecoveryHDUpdater --sign "$SIGNINGID" ~/Desktop/RecoverHDUpdater'_${OSVER}_${OSBUILD}'.pkg
else
pkgbuild --root "$payloadPath" --identifier $PKGID --scripts "$scriptPath" --version '$OSVER' --install-location /private/tmp/RecoveryHDUpdater ~/Desktop/RecoverHDUpdater'_${OSVER}_${OSBUILD}'.pkg
fi

echo "Cleaning Up... Exit Code: $?"
rm -rf "$buildPath"
echo "Finished."
'

#echo these out to their places
#put script in folder root
echo "$RecoveryHDUpdaterScript" > "$IMAGETEMP"/"RecoveryHD Updater.command"
#set permissions
chmod ugo+x "$IMAGETEMP"/"RecoveryHD Updater.command"

echo "$RecoveryHDUpdaterPKGScript" > "$IMAGETEMP"/"RecoveryHD PKG Maker.command"
#set permissions
chmod ugo+x "$IMAGETEMP"/"RecoveryHD PKG Maker.command"
}

#############
# MAIN SCRIPT
#############
#clear;

if [ "$1" == "-h" ]; then
echo "Script function:
This script will create DMG that is able to create/update a recovery partition for any given disk

Usage:
Run with no arguments to be prompted for files and paths

Or run with arguments for automatic disk image creation:
$(basename $0) RecoveryUpdatePath InstallOSXApp DMGDestinationPath
"
exit 0
fi

#get Recovery Update dmg path
if [ "$1" == "" ]; then


	while [ -z "$RECOVERYHDUPDATE" ]; do
		echo -n "Please drag in RecoveryHDUpdate.dmg (or .pkg) and press enter (or type \"download\" to do so): "
		read RECOVERYHDUPDATE
	done
else
	RECOVERYHDUPDATE="$1"
fi

if [ "$RECOVERYHDUPDATE" == "download" ]; then 
echo "Downloading in Safari..."
open -b com.apple.safari http://support.apple.com/downloads/DL1464/en_US/RecoveryHDUpdate.dmg
unset RECOVERYHDUPDATE;
while [ -z "$RECOVERYHDUPDATE" ]; do
	echo -n "Please drag in RecoveryHDUpdate.dmg once downloaded and press Enter: "
	read RECOVERYHDUPDATE
done
fi

if [ ! -f "$RECOVERYHDUPDATE" ]; then 
echo "$RECOVERYHDUPDATE not found"; 
exit
fi

#get Lion Installer path
if [ "$2" == "" ]; then
echo -n "Please drag in your \"Install OS X.app\" and press enter: "
while [ -z "$OSPATH" ]; do
read OSPATH
done
if [ ! -d "$OSPATH" ]; then echo "$OSPATH not found"; exit; fi 
else
OSPATH="$2"
fi

#get destination path for disk image to be created at
if [ "$3" == "" ]; then
echo -n "Please drag in Destination folder for disk image and press enter: "
while [ -z "$DEST" ]; do
read DEST
done

#eval to expand ~ path if typed in
#eval DEST="$DEST"

if [ ! -d "$DEST" ]; then echo "$DEST is not a valid path"; exit; fi 
else
#eval to expand ~ path if typed in
#eval DEST="$3"
DEST="$3"
fi

if [ "$(basename "$RECOVERYHDUPDATE" | tail -c 4)" == dmg ]; then
	#mount Recovery Update image
	echo "Mounting Recovery HD Update disk image"
	hdiutil attach "$RECOVERYHDUPDATE" -noverify -quiet

	#set path to that of mounted DMG	
	RECOVERYHDUPDATEPATH="$RECOVERYPATH"/RecoveryHDUpdate.pkg
	
elif [ "$(basename "$RECOVERYHDUPDATE" | tail -c 4)" == pkg ]; then
	#set path to that of the pkg
	RECOVERYHDUPDATEPATH="$RECOVERYHDUPDATE"
fi

#expand packge inside to temp folder (contains dmtest)
echo "Expanding RecoveryHDUpdate.pkg"
pkgutil --expand "$RECOVERYHDUPDATEPATH" "$RECOVERY_EXPANDED"

#if we were using what's inside the recovery update we would mount this
#RecoveryHDMeta="/Volumes/Recovery HD Update"
#hdiutil attach "$RECOVERY_EXPANDED"/RecoveryHDUpdate.pkg/RecoveryHDMeta.dmg

#open install ESD image for the newest files
echo "Mounting InstallESD.dmg"
hdiutil attach "$OSPATH"/Contents/SharedSupport/InstallESD.dmg -noverify -quiet

#10.9 changes ESD volume name
if [ ! -d "$ESDPATH" ]; then
	#try the new path
	ESDPATH="/Volumes/OS X Install ESD/"
	
	#10.9 keeps SystemVersion in BaseSystem.dmg now
	if [ -f "$ESDPATH"/BaseSystem.dmg ]; then
		hdiutil attach "$ESDPATH"/BaseSystem.dmg -noverify -quiet
	fi
	
	OSVER=$(defaults read "$BASESYSTEMPATH"/System/Library/CoreServices/SystemVersion ProductUserVisibleVersion)
	OSBUILD=$(defaults read "$BASESYSTEMPATH"/System/Library/CoreServices/SystemVersion ProductBuildVersion)
else
	#get OS version from (Mountain) Lion Installer ESD
	OSVER=$(defaults read "$ESDPATH"/System/Library/CoreServices/SystemVersion ProductUserVisibleVersion)
	OSBUILD=$(defaults read "$ESDPATH"/System/Library/CoreServices/SystemVersion ProductBuildVersion)
fi

#append info to path so disk image volume name is informative and useful
IMAGETEMP="$IMAGETEMP $OSVER $OSBUILD"

#make work the folder
mkdir -p "$IMAGETEMP"/etc/
mkdir "$IMAGETEMP"/bin/

#copy dmtest to IMAGETEMP
if [ -f "$RECOVERY_EXPANDED"/RecoveryHDUpdate.pkg/Scripts/Tools/dmtest ]; then
cp "$RECOVERY_EXPANDED"/RecoveryHDUpdate.pkg/Scripts/Tools/dmtest "$IMAGETEMP"/bin/
else
echo "$RECOVERY_EXPANDED/RecoveryHDUpdate.pkg/Scripts/Tools/dmtest not found, exiting"
exit 1
fi

#copy the BaseSystem dmg and chunklist to destination/etc
if [ -f "$ESDPATH"/BaseSystem.chunklist -o -f "$ESDPATH"/BaseSystem.dmg  ]; then
cp "$ESDPATH"/BaseSystem.chunklist "$ESDPATH"/BaseSystem.dmg "$IMAGETEMP"/etc/
#unhide BaseSystem
chflags -R nohidden "$IMAGETEMP"
else
echo "$ESDPATH/BaseSystem* not found, exiting"
fi

#have scripts made now that OSVER is known
createRecoveryScripts

#create disk image from folder
echo "Creating RecoveryHDUpdater disk image, please wait"
hdiutil create -srcfolder "$IMAGETEMP" "$DEST"/RecoveryHDUpdater_"$OSVER"_"$OSBUILD".dmg
if [ $? -eq 0 ]; then
echo "Success! Created: $DEST/RecoveryHDUpdater_"$OSVER"_"$OSBUILD".dmg"
echo "Now opening "$DEST"/RecoveryHDUpdater_"$OSVER"_"$OSBUILD".dmg"
hdiutil attach "$DEST"/RecoveryHDUpdater_"$OSVER"_"$OSBUILD".dmg
else
echo "Disk Image failed"
fi

echo "Cleaning Up"
#delete temp folders
rm -rf "$IMAGETEMP" "$RECOVERY_EXPANDED"

#eject the volumes
[ -d "$RECOVERYPATH" ] && hdiutil eject "$RECOVERYPATH"
hdiutil eject "$ESDPATH"

echo "Done."

exit